% .:
% Mesh = GmshToMatlab('SimpleQuadMesh.msh');
classdef GmshToMatlab < handle
    properties (Access = public)
        format
        numberOfPhysicalNames
        physicalNames
        numberOfNodes
        nodes
        numberOfElements
        elements
    end
    methods (Access = public)
        %    .
        function obj = GmshToMatlab(filename)
            %   . -1   .
            fid = fopen(filename,'r');
            %  .  10   .
            attempts = 1;
            while attempts~=10
                attempts = attempts+1;
                %  .
                line = fgetl(fid);
                %   ,   .   .
                switch line
                    case '$MeshFormat'
                        obj.ReadMeshFormat(fid);
                    case '$PhysicalNames'
                        obj.ReadPhysicalNames(fid);
                    case '$Nodes'
                        obj.ReadNodes(fid);
                    case '$Elements'
                        obj.ReadElements(fid);
                end
            end
        end
        
        %   "MeshFormat".
        function ReadMeshFormat(this, fid)
            %   
            line = fgetl(fid);
            formatNums = sscanf(line,'%f');
            this.format = formatNums;
            %     ( ).
            fgetl(fid);
        end
        
        %   "PhysicalNames"
        function ReadPhysicalNames(this, fid)
            %   .
            line = fgetl(fid);
            pnNum = sscanf(line,'%d');
            this.numberOfPhysicalNames = pnNum;
            %  .
            for i=1:pnNum
                line = fgetl(fid);
                pnStrData = split(line);
                pnType = sscanf(pnStrData{1}, '%d');
                pnNumber = sscanf(pnStrData{2}, '%d');
                pnNameStr = pnStrData{3};
                %  .
                pnNameStrTruncated = pnNameStr(2:(size(pnNameStr,2)-1));
                % Cell-array     c  pnNumber.
                this.physicalNames{pnNumber}={pnType,pnNameStrTruncated};
            end
            %     ( ).
            fgetl(fid);
        end
        
        %   "Nodes"
        function ReadNodes(this, fid)
            %   .
            line = fgetl(fid);
            nodesNum = sscanf(line,'%d');
            this.numberOfNodes = nodesNum;
            %  .
            for i=1:nodesNum
                line = fgetl(fid);
                this.nodes(i,:) = sscanf(line,'%f');
            end
            %     ( ).
            fgetl(fid);
        end
        
        %   "Elements"
        function ReadElements(this, fid)
            %   .
            line = fgetl(fid);
            elementsNum = sscanf(line,'%d');
            this.numberOfElements = elementsNum;
            %  .
            for i=1:elementsNum
                line = fgetl(fid);
                %    .
                elem = sscanf(line,'%f')';
                this.elements{i,1} = elem;
            end
            %     ( ).
            fgetl(fid);
        end
    end
end


